import ctypes
import glob
import os
import subprocess
from pathlib import Path
from typing import Tuple


def _get_git_root_path() -> Path:
    """
    Gets the root path of the current git repository

    Returns:
        str: string with the root path of the current git repository
    """
    return Path(
        subprocess.Popen(
            ["git", "rev-parse", "--show-toplevel"], stdout=subprocess.PIPE
        )
        .communicate()[0]
        .rstrip()
        .decode("utf-8")
    )


def _get_latest_csv_file(path: Path, filter: str = "") -> Path:
    """
    Searches in given path for the csv file which was modified latest

    Args:
        path (Path): Path to folder where csv files, which should be checked,
            are stored
        filter (str): keyword which should be used to filter the csv files.
            Defaults to ''


    Raises:
        FileNotFoundError: Raises exception if no csv file exists in given path

    Returns:
        Path: Returns the path to the csv file which was modified latest. If
            filter argument is used it returns the last modified file which
            contains the keyword given
    """
    output_csv_path = path / f"*{filter}*.csv"

    list_of_files = glob.glob(str(output_csv_path))
    if len(list_of_files) > 0:
        latest_file = Path(max(list_of_files, key=os.path.getctime))
        return latest_file
    else:
        raise FileNotFoundError


def _get_screensize() -> Tuple[int, int]:
    """
    Returns the screen size as a tuple of width and height in pixels

    Returns:
        Tuple[int, int]: width, height
    """
    return ctypes.windll.user32.GetSystemMetrics(
        0
    ), ctypes.windll.user32.GetSystemMetrics(1)


def _create_output_folder(output_path: Path, folder_name: str) -> None:
    """
    Creates output folder for the given path.

    Args:
        output_path (Path): path to output directory
        folder_name (str): name of folder which shall be created
    """
    if not os.path.exists(output_path / Path(folder_name)):
        os.mkdir(output_path / Path(folder_name))



import logging
import platform
from pathlib import Path

import matplotlib.pyplot as plt
import pandas as pd
from matplotlib.patches import Circle

from gameart.FileHandler import FileHandlerSingleton
from gameart.utils import utils

logging.basicConfig(level=logging.INFO)


def _draw_mouse_tracks() -> None:
    """
    Draws a matlab figure with the recorded mouse movement displayed as a
    x-y-diagram. The movement will be drawn with lines and the mouse
    standstill positions will be drawn as circles, which vary in size
    according to the standstill duration on each position.
    The picture will be saved to the output folder.
    """
    file_handler = FileHandlerSingleton()
    csv_file = Path(file_handler.csv_file)
    csv_file_name = csv_file.stem
    output_path = file_handler.output_path
    total_path = output_path / csv_file

    logging.info("Read csv file...")
    data_frame_mouse_movement = pd.read_csv(total_path, index_col=0)

    if platform.system() == "Windows":
        _, ax = plt.subplots(
            figsize=(
                utils._get_screensize()[0] / 100,
                utils._get_screensize()[1] / 100,
            )
        )
    else:
        screensize_default = (1920 / 100, 1080 / 100)
        _, ax = plt.subplots(figsize=screensize_default)

    x_prev, y_prev = None, None
    count_identical_rows = 0
    radius = 10
    gray_level = 0

    for _, row in data_frame_mouse_movement.iterrows():
        x, y = row["x-position"], row["y-position"]

        if x == x_prev and y == y_prev:
            count_identical_rows += 1
        else:
            if count_identical_rows >= 3:
                # Adjust the radius based on the duration of standstill
                radius = min(100, 10 + count_identical_rows)

                color = str(gray_level / 10)

                # Draw a circle with adjusted radius
                circle = Circle((x, y), radius, color=color)
                ax.add_patch(circle)

                gray_level = (gray_level + 1) % 10

            count_identical_rows = 0

        if x_prev is not None and y_prev is not None:
            # Draw a line for mouse movement
            ax.plot([x_prev, x], [y_prev, y], color="black", linewidth=0.5)

        x_prev, y_prev = x, y

    ax.set_aspect("equal", adjustable="box")
    plt.axis("off")

    output_folder_name = "gameart_images"
    utils._create_output_folder(output_path, output_folder_name)

    file_name = f"gameart_{csv_file_name}"

    try:
        plt.savefig(f"{output_path}/{output_folder_name}/{file_name}.png")
        logging.info(
            "Saved to %s/%s/%s.png"
            % (output_path, output_folder_name, file_name)
        )
    except FileExistsError as e:
        logging.error("Error: %s - %s." % (e.filename, e.strerror))



from pathlib import Path

from gameart.FileHandler import FileHandlerSingleton


def test_csv_file():
    """
    Case 1: test if csv_file has been set
    """
    file_handler = FileHandlerSingleton()
    file_handler.csv_file = "data.csv"
    assert file_handler.csv_file == "data.csv"


def test_output_path():
    """
    Case 2: test if output_path has been set
    """
    file_handler = FileHandlerSingleton()
    output_path = Path("output")
    file_handler.output_path = output_path
    assert file_handler.output_path == output_path


def test_singleton_instance():
    """
    Case 3: test that there is only one instance
    """
    file_handler1 = FileHandlerSingleton()
    file_handler2 = FileHandlerSingleton()
    assert file_handler1 == file_handler2



import fire

from gameart.api import draw_mouse_tracks, record_mouse, start_gui


def main():
    """
    Main method of the command line interface. Provides the interface to all
    public methods which can be used by the user of this python package
    """
    fire.Fire(
        {
            "record": record_mouse,
            "draw": draw_mouse_tracks,
            "start_gui": start_gui,
        }
    )


if __name__ == "__main__":
    main()



import logging
import shutil
from collections.abc import Generator
from pathlib import Path
from typing import Any

import pytest

from gameart.utils import utils


@pytest.fixture
def temp_directory(tmp_path: Path) -> Generator[Path, Any, Any]:
    """
    Creates a temporary directory for testing. And deletes it afterwards.

    Args:
        tmp_path (Path): Path to temporary directory where csv files should be
        stored to

    Yields:
        Path: Generator[Path, Any, Any]
    """
    temp_dir = tmp_path / "csv_test_dir"
    temp_dir.mkdir()
    yield temp_dir

    try:
        shutil.rmtree(temp_dir)
    except OSError as e:
        logging.error("Error: %s - %s." % (e.filename, e.strerror))


def test_get_latest_csv_file_no_files(temp_directory: Path) -> None:
    """
    Case 1: Test when no CSV files exist in the directory.

    Args:
        temp_directory (Path): Path to temporary directory where csv files are
        stored
    """
    with pytest.raises(FileNotFoundError):
        utils._get_latest_csv_file(temp_directory)


def test_get_latest_csv_file_single_file(temp_directory: Path) -> None:
    """
    Case 2: Create a single CSV file in the directory.

    Args:
        temp_directory (Path): Path to temporary directory where csv files are
        stored
    """
    csv_file = temp_directory / "test.csv"
    csv_file.touch()

    latest_file = utils._get_latest_csv_file(temp_directory)
    assert latest_file == csv_file


def test_get_latest_csv_file_multiple_files(temp_directory: Path) -> None:
    """
    Case 3: Create multiple CSV files with different modification times.

    Args:
        temp_directory (Path): Path to temporary directory where csv files are
        stored
    """
    csv_file1 = temp_directory / "file1.csv"
    csv_file1.touch()

    csv_file2 = temp_directory / "file2.csv"
    csv_file2.touch()

    # Sleep briefly to ensure file2 has a later modification time.
    import time

    time.sleep(0.1)

    csv_file3 = temp_directory / "file3.csv"
    csv_file3.touch()

    latest_file = utils._get_latest_csv_file(temp_directory)
    assert latest_file == csv_file3


def test_get_latest_csv_file_with_filter(
    temp_directory: Path,
) -> None:
    """
    Case 4: Create two CSV file in the directory. One with and one without a
    keyword. Checks if the file with the given filter argument is selected.

    Args:
        temp_directory (Path): Path to temporary directory where csv files are
        stored
    """
    csv_file1 = temp_directory / "test.csv"
    csv_file1.touch()

    csv_file2 = temp_directory / "test_keyword.csv"
    csv_file2.touch()

    latest_file = utils._get_latest_csv_file(temp_directory, "keyword")
    assert latest_file == csv_file2



import logging

logging.basicConfig(level=logging.INFO)


def record_mouse() -> None:
    """
    Public function to start recording of mouse input
    """
    from gameart.inputs import mousetracker

    mousetracker._record_mouse()


def draw_mouse_tracks(csv_file: str = "") -> None:
    """
    Public function to draw mousetracking-art based on csv file input
    which was generated by the method 'record_mouse'

    Args:
        csv_file (str): argument to provide path to csv file.
    """
    from gameart.draw import art
    from gameart.FileHandler import FileHandlerSingleton

    file_handler = FileHandlerSingleton()

    if csv_file:
        file_handler.csv_file = csv_file

    if file_handler.csv_file:
        art._draw_mouse_tracks()
    else:
        logging.error(
            """No csv file selected! Either select one (GUI) or specify the
            csv file with `gameart record --csv-file path/to/csv/file` (CLI)"""
        )


def start_gui() -> None:
    """
    Public function to start the GUI of gameart.
    """
    from gameart import gui

    gui._start_gui()



import ctypes
import datetime
import logging
import threading
import time
from typing import Any

import pandas as pd
from pynput import mouse

from gameart.FileHandler import FileHandlerSingleton
from gameart.utils import utils

# Ensuring consistent coordinates between listener and controller on Windows
PROCESS_PER_MONITOR_DPI_AWARE = 2
ctypes.windll.shcore.SetProcessDpiAwareness(PROCESS_PER_MONITOR_DPI_AWARE)

data_frame_mouse_movement = pd.DataFrame(columns=["x-position", "y-position"])
data_frame_mutex = threading.Lock()

record_thread = None
exit_event = threading.Event()


def _record_positions(fps: int) -> None:
    """
    Saves the coordinates of the current mouse position every 1/fps seconds to
    a pandas dataframe.

    Args:
        fps (int): how many times per second should the mouse position be
        saved to the pandas data_frame
    """
    global data_frame_mouse_movement
    while not exit_event.is_set():
        with data_frame_mutex:
            mouse_position = mouse.Controller().position
            x, y = mouse_position[0], mouse_position[1]

            y = abs(
                y - utils._get_screensize()[1]
            )  # flip y value, so that bottom of monitor is 0
            logging.info("Pointer moved to {0}".format((x, y)))

            data_frame_mouse_movement.loc[
                len(data_frame_mouse_movement.index)
            ] = [  # type: ignore
                x,
                y,
            ]

        time.sleep(1 / fps)


def _on_click(
    x: float, y: float, button: mouse.Button, pressed: bool, **kwargs
) -> Any:
    """
    Method is called when a mouse button is clicked. Stops recording on
    right mouse button click.
    Therefore it sets the exit event for the record_thread.

    Args:
        x (float): x-coordinate of the current mouse position
        y (float): y-coordinate of the current mouse position
        button (mouse.Button): mouse button which was clicked
        pressed (bool): is mouse button currently pressed?

    Returns:
        Any: returns False when right mouse button was clicked,
            for all other cases method returns None
    """
    global data_frame_mouse_movement

    logging.info(
        "{0} at {1}".format("Pressed" if pressed else "Released", (x, y))
    )
    if pressed and button == mouse.Button.right:
        logging.info("Recording stopped")

        exit_event.set()

        current_time = datetime.datetime.now()
        formatted_time = current_time.strftime("%Y-%m-%d_%H-%M-%S")
        file_name = f"mousetracker_{formatted_time}.csv"

        file_handler = FileHandlerSingleton()
        output_path = file_handler.output_path
        file_handler.csv_file = file_name
        total_path = output_path / file_name

        data_frame_mouse_movement.to_csv(total_path, sep=",", encoding="utf-8")

        logging.info("Log saved to %s" % (total_path))

        return False


def _record_mouse() -> None:
    """
    Starts the listener of pynput to record mouse movement and buttons.
    Uses threading to both detect pynput listener and controller at the same
    time.
    """
    logging.info("Recording started")
    logging.info("Press Right Mouse Button to stop recording")
    logging.info("Recording...")

    listener = mouse.Listener(on_click=_on_click)
    listener.start()

    global record_thread
    record_thread = threading.Thread(target=_record_positions(30))
    record_thread.daemon = True
    record_thread.start()



import importlib.metadata
import logging
import sys
import webbrowser
from pathlib import Path

from PyQt5.QtCore import QDir, QModelIndex, Qt
from PyQt5.QtWidgets import (
    QAction,
    QApplication,
    QFileDialog,
    QFileSystemModel,
    QGridLayout,
    QLabel,
    QListView,
    QMainWindow,
    QMenu,
    QPushButton,
    QTabWidget,
    QWidget,
)

from gameart.api import draw_mouse_tracks, record_mouse
from gameart.FileHandler import FileHandlerSingleton


class MainWindow(QMainWindow):
    """
    Class of Main Window. Consists of a menu bar and a tab widget.
    It also uses the singleton file handler class of gameart to get and set the
    relevant paths. Class has the property base_folder to get and set the base
    folder which gets displayed in the list view.

    Args:
        QMainWindow (object): The QMainWindow class provides a main
        application window.
    """

    def __init__(self) -> None:
        super().__init__()
        self.file_handler = FileHandlerSingleton()
        self._base_folder = str(self.file_handler.output_path)

        self.setWindowTitle(
            f"GameArt - {importlib.metadata.version('gameart')}"
        )
        self.resize(500, 500)

        menu_bar = self.menuBar()
        self._create_actions()
        menu_bar.addMenu(self._menu_bar())  # type: ignore

        self.tabWidget = QTabWidget()
        self.setCentralWidget(self.tabWidget)
        self.main_tab = QWidget()
        self._setup_main_tab()
        self.tabWidget.addTab(self.main_tab, "Record and Generate")

    @property
    def base_folder(self) -> str:
        return self._base_folder

    @base_folder.setter
    def base_folder(self, value: str):
        self._base_folder = value

    def _create_actions(self) -> None:
        """
        Creates actions for the menu bar buttons. Pressing the button
        "Select Output Folder" connects to the _select_output_folder method
        """
        self.output_folder_action = QAction(self)
        self.output_folder_action.setText("&Select Output Folder")
        self.output_folder_action.triggered.connect(self._select_output_folder)

    def _menu_bar(self) -> QMenu:
        """
        Creates the menus for the menu bar and connects the respective actions
        - File

        Returns:
            QMenu: Returns the PyQt5 QMenu object
        """
        file_menu = QMenu("&File", self)
        file_menu.addAction(self.output_folder_action)

        return file_menu

    def _setup_main_tab(self) -> None:
        """
        Sets up the main tab by creating buttons, file system and list view.
        It adds all relevant widgets to the layout of the tab.
        """
        layout = QGridLayout()

        # Create buttons
        btn_select_base_folder = QPushButton("Select Folder to View")
        btn_select_base_folder.clicked.connect(self._select_base_folder)
        layout.addWidget(btn_select_base_folder, 1, 0)

        btn_start_record = QPushButton("Start Recording")
        btn_start_record.clicked.connect(self._execute_record)
        layout.addWidget(btn_start_record, 3, 0)

        btn_generate_image = QPushButton("Generate Image")
        btn_generate_image.clicked.connect(self._execute_draw)
        layout.addWidget(btn_generate_image, 4, 0)

        btn_open_upload_page = QPushButton("Upload Image (coming soon)")
        btn_open_upload_page.clicked.connect(self._open_upload_page)
        btn_open_upload_page.setEnabled(False)
        layout.addWidget(btn_open_upload_page, 6, 0)

        # Create labels
        self.label_current_output_folder = QLabel(
            "label_current_output_folder"
        )
        self.label_current_output_folder.setAlignment(
            Qt.AlignmentFlag.AlignCenter
        )
        self._update_output_folder_label(str(self.file_handler.output_path))
        layout.addWidget(self.label_current_output_folder, 5, 0)

        # Create file system model
        self.file_model = QFileSystemModel()
        self.file_model.setRootPath(QDir.rootPath())
        self.file_model.setFilter(
            QDir.NoDotAndDotDot | QDir.Files | QDir.AllDirs
        )
        self.file_model.setNameFilters(["*.csv"])

        # Create list view
        self.list_view = QListView()
        self.list_view.setModel(self.file_model)
        self._update_list_view(self.base_folder)
        self.list_view.clicked.connect(self._on_list_view_clicked)
        layout.addWidget(self.list_view, 2, 0)

        self.main_tab.setLayout(layout)

    def _update_list_view(self, folder: str) -> None:
        """
        Updates the list view by setting the root index to the given folder
        path

        Args:
            folder (str): Path to the folder which shall be visible in list
            view
        """
        self.list_view.setRootIndex(self.file_model.setRootPath(folder))
        logging.info("List view updated.")

    def _update_output_folder_label(self, folder: str) -> None:
        """
        Updates the label of the current output folder by setting the text to
        the given folder path

        Args:
            folder (str): Path to the folder which shall be visible in label
            view
        """
        self.label_current_output_folder.setText(
            f"(Current output folder {folder}, File -> "
            + "Select Output Folder)"
        )
        logging.info("Label of output folder updated.")

    def _on_list_view_clicked(self, index: QModelIndex) -> None:
        """
        Gets the file path of the selected file and sets the csv_file property
        of the file handler class accordingly. If the file is not a .csv the
        method returns None.

        Args:
            index (QModelIndex): Index of the file which got selected in list
            view
        """
        file_extension = self.file_model.fileInfo(index).completeSuffix()
        if file_extension != "csv":
            logging.warning(
                "File extension .%s not supported!" % (file_extension)
            )
            return
        file_path = self.file_model.fileInfo(index).absoluteFilePath()
        logging.info("File selected %s" % (file_path))
        self.file_handler.csv_file = file_path

    def _select_base_folder(self) -> None:
        """
        Method to set the base folder of the list view. It updates the list
        view to display all files in that base folder
        """
        self.base_folder = QFileDialog.getExistingDirectory(
            self, "Select Folder"
        )
        if self.base_folder:
            self._update_list_view(self.base_folder)
            logging.info("Folder selected %s" % (self.base_folder))
        else:
            logging.warning("No folder selected!")

    def _select_output_folder(self) -> None:
        """
        Opens a file dialog to let the user select the output folder, where the
        gameart output folder will be created after generating the images.
        """
        output_folder = QFileDialog.getExistingDirectory(
            self, "Select Output Folder"
        )
        if output_folder:
            self.base_folder = output_folder
            self._update_output_folder_label(output_folder)
            self._update_list_view(output_folder)
            self.file_handler.output_path = Path(output_folder)
            logging.info("Output folder selected %s" % (output_folder))
        else:
            logging.warning("No output folder selected!")

    def _execute_record(self) -> None:
        """
        Executes the record mouse method and updates the list view afterwards.
        """
        record_mouse()
        self._update_list_view(self.base_folder)

    def _execute_draw(self) -> None:
        """
        Executes the draw method and updates the list view afterwards.
        """
        draw_mouse_tracks()
        self._update_list_view(self.base_folder)

    def _open_upload_page(self) -> None:
        """
        Opens the upload page of gameart_web
        """
        webbrowser.open("http://127.0.0.1:8000/dashboard/")


def _start_gui() -> None:
    """
    Starts the GUI.
    """
    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()

    sys.exit(app.exec_())


# entry point for pyinstaller
if __name__ == "__main__":
    _start_gui()



from pathlib import Path


class FileHandlerSingleton:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self):
        if not hasattr(self, "initialized"):
            self._csv_file = ""
            self._output_path = Path(".").absolute()
            self.initialized = True

    @property
    def csv_file(self) -> str:
        return self._csv_file

    @csv_file.setter
    def csv_file(self, value: str):
        self._csv_file = value

    @property
    def output_path(self) -> Path:
        return self._output_path

    @output_path.setter
    def output_path(self, value: Path):
        self._output_path = value
